﻿using Dapper;
using DapperStart.Models;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Options;
using System.Data.SqlClient;
using System.Data;

namespace DapperStart.Controllers
{
    public class ExecuteController : Controller
    {
        private IOptions<Connections> cs;

        public ExecuteController(IOptions<Connections> cs)
        {
            this.cs = cs;
        }

        public IActionResult InsertSingle()
        {
            string myCS = cs.Value.DefaultConnection;

            var sql = @"Insert into Inventory(Name, Price, Quantity) values (@Name, @Price, @Quantity)";

            using (var connection = new SqlConnection(myCS))
            {
                var rowsAffected = connection.Execute(sql, new { Name = "Shirts", Price = 19, Quantity = 20 });
            }

            return RedirectToAction("Index", "Home");
        }

        public IActionResult InsertMultiple()
        {
            string myCS = cs.Value.DefaultConnection;

            var sql = @"Insert into Inventory(Name, Price, Quantity) values (@Name, @Price, @Quantity)";

            using (var connection = new SqlConnection(myCS))
            {
                var rowsAffected = connection.Execute(sql,
                    new[]
                    {
                        new { Name = "Shirts", Price = 19, Quantity = 20  },
                        new { Name = "Pants", Price = 25, Quantity = 30  }
                    });
            }

            return RedirectToAction("Index", "Home");
        }

        public IActionResult InsertSingleDP()
        {
            string myCS = cs.Value.DefaultConnection;

            var sql = @"Insert into Inventory(Name, Price, Quantity) values (@Name, @Price, @Quantity)";

            using (var connection = new SqlConnection(myCS))
            {
                DynamicParameters parameter = new DynamicParameters();
                parameter.Add("@Name", "Shirts", DbType.String, ParameterDirection.Input);
                parameter.Add("@Price", 19, DbType.Double, ParameterDirection.Input);
                parameter.Add("@Quantity", 20, DbType.Int32, ParameterDirection.Input);

                var rowsAffected = connection.Execute(sql, parameter);
            }

            return RedirectToAction("Index", "Home");
        }

        public IActionResult InsertMultipleDP()
        {
            string myCS = cs.Value.DefaultConnection;

            var sql = @"Insert into Inventory(Name, Price, Quantity) values (@Name, @Price, @Quantity)";

            using (var connection = new SqlConnection(myCS))
            {
                var parameterList = new List<DynamicParameters>();

                DynamicParameters parameter = new DynamicParameters();
                parameter.Add("@Name", "Shirts", DbType.String, ParameterDirection.Input);
                parameter.Add("@Price", 19, DbType.Double, ParameterDirection.Input);
                parameter.Add("@Quantity", 20, DbType.Int32, ParameterDirection.Input);

                parameterList.Add(parameter);

                parameter.Add("@Name", "Pants", DbType.String, ParameterDirection.Input);
                parameter.Add("@Price", 25, DbType.Double, ParameterDirection.Input);
                parameter.Add("@Quantity", 30, DbType.Int32, ParameterDirection.Input);

                parameterList.Add(parameter);

                var rowsAffected = connection.Execute(sql, parameterList);
            }

            return RedirectToAction("Index", "Home");
        }

        public IActionResult UpdateSingle()
        {
            string myCS = cs.Value.DefaultConnection;

            var sql = @"Update Inventory SET Name = @Name, Price = @Price, Quantity = @Quantity Where Id = @Id";

            using (var connection = new SqlConnection(myCS))
            {
                var rowsAffected = connection.Execute(sql, new { Name = "Pants", Price = 18, Quantity = 25, Id = 1 });
            }

            return RedirectToAction("Index", "Home");
        }

        public IActionResult UpdateMultiple()
        {
            string myCS = cs.Value.DefaultConnection;

            var sql = @"Update Inventory SET Name = @Name, Price = @Price, Quantity = @Quantity Where Id = @Id";

            using (var connection = new SqlConnection(myCS))
            {
                var rowsAffected = connection.Execute(sql,
                    new[]
                    {
                        new { Name = "Pants", Price = 18, Quantity = 25, Id = 1 },
                        new { Name = "Skirts", Price = 26, Quantity = 35, Id = 2 }
                    });
            }

            return RedirectToAction("Index", "Home");
        }

        public IActionResult DeleteSingle()
        {
            string myCS = cs.Value.DefaultConnection;

            var sql = @"Delete From Inventory Where Id = @Id";

            using (var connection = new SqlConnection(myCS))
            {
                var rowsAffected = connection.Execute(sql, new { Id = 1 });
            }

            return RedirectToAction("Index", "Home");
        }

        public IActionResult DeleteMultiple()
        {
            string myCS = cs.Value.DefaultConnection;

            var sql = @"Delete From Inventory Where Id = @Id";

            using (var connection = new SqlConnection(myCS))
            {
                var rowsAffected = connection.Execute(sql,
                    new[]
                    {
                        new { Id = 1 },
                        new { Id = 2 }
                    });
            }

            return RedirectToAction("Index", "Home");
        }

        public IActionResult DeleteInCommand()
        {
            string myCS = cs.Value.DefaultConnection;

            var sql = @"Delete From Inventory Where Id IN @Ids";

            using (var connection = new SqlConnection(myCS))
            {
                var ids = new[] { 1, 2, 3 };
                var rowsAffected = connection.Execute(sql, new { Ids = ids });
            }

            return RedirectToAction("Index", "Home");
        }

        public IActionResult InsertSingleSP()
        {
            string myCS = cs.Value.DefaultConnection;

            using (var connection = new SqlConnection(myCS))
            {
                var rowsAffected = connection.Execute("CreateInventory", new { Name = "Skirts", Price = 29, Quantity = 100 }, commandType: CommandType.StoredProcedure);
            }

            return RedirectToAction("Index", "Home");
        }

        public IActionResult InsertMultipleSP()
        {
            string myCS = cs.Value.DefaultConnection;

            using (var connection = new SqlConnection(myCS))
            {
                var rowsAffected = connection.Execute("CreateInventory",
                    new[]
                    {
                        new { Name = "Skirts", Price = 29, Quantity = 100 },
                        new { Name = "Coats", Price = 69, Quantity = 200 },
                        new { Name = "Caps", Price = 9, Quantity = 400 }
                    },
                commandType: CommandType.StoredProcedure);
            }

            return RedirectToAction("Index", "Home");
        }

        public IActionResult Scalar()
        {
            string myCS = cs.Value.DefaultConnection;

            var sql = "SELECT Count(*) FROM Inventory";

            using (var connection = new SqlConnection(myCS))
            {
                var count = connection.ExecuteScalar<int>(sql);
            }

            /*var sql1 = "SELECT Name FROM Inventory";

            using (var connection = new SqlConnection(myCS))
            {
                var name = connection.ExecuteScalar<string>(sql1);
            }*/

            /*var sql2 = "SELECT SUM(Price) FROM Inventory";

            using (var connection = new SqlConnection(myCS))
            {
                var sum = connection.ExecuteScalar<int>(sql2);
            }*/

            return RedirectToAction("Index", "Home");
        }

        public IActionResult Reader()
        {
            string myCS = cs.Value.DefaultConnection;

            //From Query
            using (var connection = new SqlConnection(myCS))
            {
                var reader = connection.ExecuteReader("SELECT * FROM Inventory");

                DataTable table = new DataTable();
                table.Load(reader);
            }

            // From Stored Procedure
            /*using (var connection = new SqlConnection(myCS))
            {
                var reader = connection.ExecuteReader("SelectInventory @Price=@price", new { price = 9000 });

                DataTable table = new DataTable();
                table.Load(reader);
            }*/
            return RedirectToAction("Index", "Home");
        }

        public async Task<IActionResult> InsertSingleA()
        {
            string myCS = cs.Value.DefaultConnection;

            var sql = @"Insert into Inventory(Name, Price, Quantity) values (@Name, @Price, @Quantity)";

            using (var connection = new SqlConnection(myCS))
            {
                var customers = await connection.ExecuteAsync(sql, new { Name = "Shoes", Price = 99, Quantity = 200 });
            }

            return RedirectToAction("Index", "Home");
        }

        public IActionResult Transaction()
        {
            string myCS = cs.Value.DefaultConnection;

            var sql = @"Insert into Inventory(Name, Price, Quantity) values (@Name, @Price, @Quantity)";

            using (var connection = new SqlConnection(myCS))
            {
                connection.Open();
                using (var transaction = connection.BeginTransaction())
                {
                    var affectedRows = connection.Execute(sql, new { Name = "x1", Price = 1, Quantity = 2 }, transaction: transaction);

                    transaction.Commit();
                }
            }

            return View();
        }

    }
}
