﻿using DapperStart.Models;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Options;
using System.Data.SqlClient;
using System.Data;
using Dapper;

namespace DapperStart.Controllers
{
    public class ReadController : Controller
    {
        private IOptions<Connections> cs;

        public ReadController(IOptions<Connections> cs)
        {
            this.cs = cs;
        }
        public IActionResult Query()
        {
            string myCS = cs.Value.DefaultConnection;
            string query = "SELECT * FROM Inventory";

            using (var connection = new SqlConnection(myCS))
            {
                var result = connection.Query<Inventory>(query).ToList();
            }

            return RedirectToAction("Index", "Home");
        }
        public IActionResult QueryParameter()
        {
            string myCS = cs.Value.DefaultConnection;
            string query = "SELECT * FROM Inventory where id in @Ids";
            var ids = new[] { 1, 2, 3 };

            using (var connection = new SqlConnection(myCS))
            {
                var result = connection.Query<Inventory>(query, new { Ids = ids }).ToList();
            }

            return RedirectToAction("Index", "Home");
        }

        public async Task<IActionResult> QueryParameterA()
        {
            string myCS = cs.Value.DefaultConnection;
            string query = "SELECT * FROM Inventory WHERE Id = @Id";

            using (var connection = new SqlConnection(myCS))
            {
                var result = await connection.QueryAsync(query, new { Id = 1 });
            }

            return RedirectToAction("Index", "Home");
        }

        public IActionResult QuerySP()
        {
            string myCS = cs.Value.DefaultConnection;

            using (var connection = new SqlConnection(myCS))
            {
                var result = connection.Query<Inventory>("SelectInventory", new { Price = 25 }, commandType: CommandType.StoredProcedure).ToList();
            }

            return RedirectToAction("Index", "Home");
        }

        public IActionResult QueryM()
        {
            string myCS = cs.Value.DefaultConnection;
            string query = "SELECT * FROM Inventory;";


            using (var connection = new SqlConnection(myCS))
            {
                var result = connection.Query<Inventory>(query).ToList();
            }

            return RedirectToAction("Index", "Home");
        }

        public IActionResult QueryMultiple()
        {
            string myCS = cs.Value.DefaultConnection;
            string query = "SELECT * FROM Inventory where Id=@Id; Select * From Employee where Name=@Name";


            using (var connection = new SqlConnection(myCS))
            {
                using (var multi = connection.QueryMultiple(query, new { Id = 1, Name = "Yogi" }))
                {
                    var inventory = multi.Read<Inventory>().First();
                    var employee = multi.Read<Employee>().ToList();
                }
            }

            return RedirectToAction("Index", "Home");
        }

        public IActionResult QueryFD()
        {
            string myCS = cs.Value.DefaultConnection;
            string query = "SELECT * FROM Inventory;";

            using (var connection = new SqlConnection(myCS))
            {
                var result = connection.QueryFirstOrDefault<Inventory>(query);
            }

            return RedirectToAction("Index", "Home");
        }
    }
}
