﻿using Dapper;
using DapperStart.Models;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Hosting.Internal;
using Microsoft.Extensions.Options;
using System.Data.SqlClient;

namespace DapperStart.Controllers
{
    public class SchoolController : Controller
    {
        private IOptions<Connections> cs;

        public SchoolController(IOptions<Connections> cs)
        {
            this.cs = cs;
        }

        public IActionResult ReadSSD()
        {
            string myCS = cs.Value.DefaultConnection;

            string sql = "SELECT * FROM School AS A LEFT JOIN SchoolDetail AS B ON A.Id = B.SchoolId;";

            List<School> schools;
            using (var connection = new SqlConnection(myCS))
            {

                schools = connection.Query<School, SchoolDetail, School>(
                        sql,
                        (school, schoolDetail) =>
                        {
                            school.SchoolDetail = schoolDetail;
                            return school;
                        },
                        splitOn: "SchoolId")
                    .Distinct()
                    .ToList();
            }

            return View(schools);
        }

        public IActionResult ReadST()
        {
            string myCS = cs.Value.DefaultConnection;

            string sql = "SELECT * FROM School AS A LEFT JOIN Teacher AS B ON A.Id = B.SchoolId;";

            List<School> schools;
            using (var connection = new SqlConnection(myCS))
            {
                var dictionary = new Dictionary<int, School>();


                schools = connection.Query<School, Teacher, School>(
                sql,
                (school, teacher) =>
                {
                    School e;

                    if (!dictionary.TryGetValue(school.Id, out e))
                    {
                        e = school;
                        e.Teacher = new List<Teacher>();
                        dictionary.Add(e.Id, e);
                    }

                    e.Teacher.Add(teacher);
                    return e;
                },
                splitOn: "Id")
                .Distinct()
                .ToList();
            }

            return View(schools);
        }

        public IActionResult Create()
        {
            return View();
        }

        [HttpPost]
        public IActionResult Create(School school)
        {
            if (ModelState.IsValid)
            {
                string myCS = cs.Value.DefaultConnection;

                var sql = @"Insert into School(Name, City) values (@Name, @City)";

                using (var connection = new SqlConnection(myCS))
                {
                    var rowsAffected = connection.Execute(sql, new { Name = school.Name, City = school.City });
                }

                return RedirectToAction("ReadSSD");
            }
            else
                return View();
        }

        public IActionResult Update(int id)
        {
            string myCS = cs.Value.DefaultConnection;
            string query = "SELECT * FROM School Where Id=@Id";

            var school = new School();
            using (var connection = new SqlConnection(myCS))
            {
                school = connection.Query<School>(query, new { Id = id }).FirstOrDefault();
            }

            return View(school);
        }

        [HttpPost]
        public IActionResult Update(School school)
        {
            if (ModelState.IsValid)
            {
                string myCS = cs.Value.DefaultConnection;

                var sql = @"Update School SET Name=@Name, City=@City WHERE Id=@Id";

                using (var connection = new SqlConnection(myCS))
                {
                    var rowsAffected = connection.Execute(sql, new { Name = school.Name, City = school.City, Id = school.Id });
                }

                return RedirectToAction("ReadSSD");
            }
            else
                return View();
        }

        [HttpPost]
        public IActionResult Delete(int id)
        {
            string myCS = cs.Value.DefaultConnection;

            var sql1 = @"Delete From SchoolDetail Where SchoolId = @Id";
            var sql2 = @"Delete From Teacher Where SchoolId = @Id";
            var sql3 = @"Delete From School Where Id = @Id";

            using (var connection = new SqlConnection(myCS))
            {
                var result1 = connection.Execute(sql1, new { Id = id });
                var result2 = connection.Execute(sql2, new { Id = id });
                var result3 = connection.Execute(sql3, new { Id = id });
            }

            return RedirectToAction("ReadSSD");
        }
    }
}
