﻿using DapperStart.Models;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Options;
using System.Data.SqlClient;
using System.Data;
using Dapper;

namespace DapperStart.Controllers
{
    public class SeedController : Controller
    {
        private IOptions<Connections> cs;

        public SeedController(IOptions<Connections> cs)
        {
            this.cs = cs;
        }

        public IActionResult Seed()
        {
            string myCS = cs.Value.DefaultConnection;

            using (var connection = new SqlConnection(myCS))
            {
                // CREATE the Table
                connection.Execute(@"
                    CREATE TABLE [Customer]
                    (
	                    [Id] [INT] IDENTITY(1,1) NOT NULL,
	                    [Name] [VARCHAR](20) NULL,
   	                    [Email] [VARCHAR](20) NULL,
	
	                    CONSTRAINT [PK_Customer] PRIMARY KEY CLUSTERED 
	                    (
		                    [Id] ASC
	                    )
                    )
		                    ");

                // CREATE the TVP type
                connection.Execute(@"
                    CREATE TYPE TVP_Customer AS TABLE
                    (
	                    [Name] [VARCHAR](20) NULL,
	                    [Email] [VARCHAR](20) NULL
                    )
		                    ");

                // CREATE the Stored Procedure			   
                connection.Execute(@"
                    CREATE PROCEDURE Customer_Seed
	                    @Customers TVP_Customer READONLY
                    AS
                    BEGIN
	                    INSERT INTO Customer (Name,Email)
	                    SELECT Name, Email
	                    FROM @Customers
                    END
		                    ");

                var dt = new DataTable();
                dt.Columns.Add("Name");
                dt.Columns.Add("Email");

                for (int i = 0; i < 5; i++)
                {
                    dt.Rows.Add("Name_" + i, "Email_" + i);
                }
                connection.Execute("Customer_Seed", new { Customers = dt.AsTableValuedParameter("TVP_Customer") }, commandType: CommandType.StoredProcedure);
            }

            return RedirectToAction("Index", "Home");
        }
    }
}
